﻿namespace ApNegociosSL.Controls
{
  using System;
  using System.Windows;
  using System.Windows.Controls;
  using System.Windows.Shapes;
  using System.Windows.Threading;

  /// <summary>
  /// Control que proporciona un indicador visual cuando una aplicación está ocupada.
  /// </summary>
  [TemplateVisualState(Name = VisualStates.StateIdle, GroupName = VisualStates.GroupBusyStatus)]
  [TemplateVisualState(Name = VisualStates.StateBusy, GroupName = VisualStates.GroupBusyStatus)]
  [TemplateVisualState(Name = VisualStates.StateVisible, GroupName = VisualStates.GroupVisibility)]
  [TemplateVisualState(Name = VisualStates.StateHidden, GroupName = VisualStates.GroupVisibility)]
  [StyleTypedProperty(Property = "OverlayStyle", StyleTargetType = typeof(Rectangle))]
  [StyleTypedProperty(Property = "ProgressBarStyle", StyleTargetType = typeof(ProgressBar))]
  public class BusyIndicator : ContentControl
  {
    /// <summary>
    /// Obtiene o establece un valor que indica si el BusyContent es visible.
    /// </summary>
    protected bool IsContentVisible { get; set; }

    /// <summary>
    /// Timer utilizado para retrasar la visualización inicial y evitar el parpadeo.
    /// </summary>
    private DispatcherTimer _displayAfterTimer;

    /// <summary>
    /// Crea una instancia de una nueva instancia del control BusyIndicator.
    /// </summary>
    public BusyIndicator()
    {
      DefaultStyleKey = typeof(BusyIndicator);
      _displayAfterTimer = new DispatcherTimer();
      _displayAfterTimer.Tick += new EventHandler(DisplayAfterTimerElapsed);
    }

    /// <summary>
    /// Invalida el método OnApplyTemplate.
    /// </summary>
    public override void OnApplyTemplate()
    {
      base.OnApplyTemplate();
      ChangeVisualState(false);
    }

    /// <summary>
    /// Controlador para el DisplayAfterTimer.
    /// </summary>
    /// <param name="sender">Remitente del evento.</param>
    /// <param name="e">Argumentos del evento.</param>
    private void DisplayAfterTimerElapsed(object sender, EventArgs e)
    {
      _displayAfterTimer.Stop();
      IsContentVisible = true;
      ChangeVisualState(true);
    }

    /// <summary>
    /// Cambia los estados visuales del control.
    /// </summary>
    /// <param name="useTransitions">True si se tienen que utilizar transiciones de estado.</param>
    protected virtual void ChangeVisualState(bool useTransitions)
    {
      VisualStateManager.GoToState(this, IsBusy ? VisualStates.StateBusy : VisualStates.StateIdle, useTransitions);
      VisualStateManager.GoToState(this, IsContentVisible ? VisualStates.StateVisible : VisualStates.StateHidden, useTransitions);
    }

    /// <summary>
    /// Obtiene o establece un valor que indica si se tiene que mostrar el indicador de ocupado.
    /// </summary>
    public bool IsBusy
    {
      get { return (bool)GetValue(IsBusyProperty); }
      set { SetValue(IsBusyProperty, value); }
    }

    /// <summary>
    /// Identifica la dependency property IsBusy.
    /// </summary>
    public static readonly DependencyProperty IsBusyProperty = DependencyProperty.Register(
        "IsBusy",
        typeof(bool),
        typeof(BusyIndicator),
        new PropertyMetadata(false, new PropertyChangedCallback(OnIsBusyChanged)));

    /// <summary>
    /// La propiedad IsBusyProperty cambió de controlador.
    /// </summary>
    /// <param name="d">BusyIndicator que cambió su IsBusy.</param>
    /// <param name="e">Argumentos del evento.</param>
    private static void OnIsBusyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      ((BusyIndicator)d).OnIsBusyChanged(e);
    }

    /// <summary>
    /// La propiedad IsBusyProperty cambió de controlador.
    /// </summary>
    /// <param name="e">Argumentos del evento.</param>
    protected virtual void OnIsBusyChanged(DependencyPropertyChangedEventArgs e)
    {
      if (IsBusy)
      {
        if (DisplayAfter.Equals(TimeSpan.Zero))
        {
          // Hacer visible
          IsContentVisible = true;
        }
        else
        {
          // Establecer la visibilidad de un temporizador
          _displayAfterTimer.Interval = DisplayAfter;
          _displayAfterTimer.Start();
        }
      }
      else
      {
        // Anular visibilidad
        _displayAfterTimer.Stop();
        IsContentVisible = false;
      }
      ChangeVisualState(true);
    }

    /// <summary>
    /// Obtiene o establece un valor que indica el contenido ocupado que se va a mostrar al usuario.
    /// </summary>
    public object BusyContent
    {
      get { return (object)GetValue(BusyContentProperty); }
      set { SetValue(BusyContentProperty, value); }
    }

    /// <summary>
    /// Identifica la dependency property BusyContent.
    /// </summary>
    public static readonly DependencyProperty BusyContentProperty = DependencyProperty.Register(
        "BusyContent",
        typeof(object),
        typeof(BusyIndicator),
        new PropertyMetadata(null));

    /// <summary>
    /// Obtiene o establece un valor que indica la plantilla que se va a utilizar para mostrar el contenido ocupado al usuario.
    /// </summary>
    public DataTemplate BusyContentTemplate
    {
      get { return (DataTemplate)GetValue(BusyContentTemplateProperty); }
      set { SetValue(BusyContentTemplateProperty, value); }
    }

    /// <summary>
    /// Identifica la dependency property BusyTemplate.
    /// </summary>
    public static readonly DependencyProperty BusyContentTemplateProperty = DependencyProperty.Register(
        "BusyContentTemplate",
        typeof(DataTemplate),
        typeof(BusyIndicator),
        new PropertyMetadata(null));

    /// <summary>
    /// Obtiene o establece un valor que indica cuánto tiempo pasará hasta que se muestre el contenido ocupado.
    /// </summary>
    public TimeSpan DisplayAfter
    {
      get { return (TimeSpan)GetValue(DisplayAfterProperty); }
      set { SetValue(DisplayAfterProperty, value); }
    }

    /// <summary>
    /// Identifica la dependency property DisplayAfter.
    /// </summary>
    public static readonly DependencyProperty DisplayAfterProperty = DependencyProperty.Register(
        "DisplayAfter",
        typeof(TimeSpan),
        typeof(BusyIndicator),
        new PropertyMetadata(TimeSpan.FromSeconds(0.1)));

    /// <summary>
    /// Obtiene o establece un valor que indica el estilo que se va a utilizar para la superposición.
    /// </summary>
    public Style OverlayStyle
    {
      get { return (Style)GetValue(OverlayStyleProperty); }
      set { SetValue(OverlayStyleProperty, value); }
    }

    /// <summary>
    /// Identifica la dependency property OverlayStyle.
    /// </summary>
    public static readonly DependencyProperty OverlayStyleProperty = DependencyProperty.Register(
        "OverlayStyle",
        typeof(Style),
        typeof(BusyIndicator),
        new PropertyMetadata(null));

    /// <summary>
    /// Obtiene o establece un valor que indica el estilo que se va a utilizar para la barra de progreso.
    /// </summary>
    public Style ProgressBarStyle
    {
      get { return (Style)GetValue(ProgressBarStyleProperty); }
      set { SetValue(ProgressBarStyleProperty, value); }
    }

    /// <summary>
    /// Identifica la dependency property ProgressBarStyle.
    /// </summary>
    public static readonly DependencyProperty ProgressBarStyleProperty = DependencyProperty.Register(
        "ProgressBarStyle",
        typeof(Style),
        typeof(BusyIndicator),
        new PropertyMetadata(null));
  }
}